package fr.ippon.direct;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.nio.MappedByteBuffer;
import java.nio.channels.FileChannel;

import com.sun.xml.internal.ws.util.ByteArrayBuffer;


/**
 * @author IPPON_NANTES
 * 
 */
public class LoadMemoryMappedFile {
	private static int NANO_TO_MILLI = 1000000;

	public static void main(String[] args) throws InterruptedException,
			IOException, ClassNotFoundException {

		Boolean firstLine = false;
		String path = "bin/ressource/villes1000hab.csv";
		try {
			long startTime = System.nanoTime();
			File file = new File(path);

			// Creation d'un memory-mapped file en lecture seule
			FileChannel roChannel = new RandomAccessFile(file, "r")
					.getChannel();
			MappedByteBuffer mbb = roChannel.map(FileChannel.MapMode.READ_ONLY,
					0, (int) roChannel.size());
			
			// tableau de bits utilisé pour la lectre
			ByteArrayBuffer bits = new ByteArrayBuffer();
			
			while(mbb.hasRemaining()) {
				// on lit le prochain octet
				byte b = mbb.get();
				if (b == 10 || b == 13) {
					if (!firstLine) {
						// On ignore la première ligne (entête)
						firstLine = true;
					} else {
						// on affiche le contenu du fichier 
						System.out.println(bits.toString());
					}
					// on passe à une nouvelle ligne 
					// on réinitialise le tableau
					bits = new ByteArrayBuffer();
				}  else
					// on ajoute l'octet au tableau
					bits.write(b);
			}
			
			long estimatedTimeTraitement = System.nanoTime() - startTime;
			System.out.println("");
			System.out.println("MemoryMappedFile");
			System.out.println("Temps de traitement (ms) : "
					+ estimatedTimeTraitement / NANO_TO_MILLI);
			
			// on ferme le canal
			roChannel.close();
			
			// on vide le buffer
			mbb.clear();
			
			// on affiche l'utilisation mémoire
			TraceMemory.trace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
}
