package fr.ippon.direct;

import java.lang.management.GarbageCollectorMXBean;
import java.lang.management.ManagementFactory;
import java.lang.management.MemoryMXBean;
import java.lang.management.MemoryPoolMXBean;
import java.util.List;
import java.util.Set;

import javax.management.AttributeNotFoundException;
import javax.management.InstanceNotFoundException;
import javax.management.MBeanException;
import javax.management.MBeanServer;
import javax.management.MalformedObjectNameException;
import javax.management.ObjectInstance;
import javax.management.ObjectName;
import javax.management.ReflectionException;

/**
 * @author IPPON_NANTES
 * 
 */
public class TraceMemory {

	private static void printGarbageCollectorMXBean() {
		List<GarbageCollectorMXBean> list = (List<GarbageCollectorMXBean>) ManagementFactory
				.getGarbageCollectorMXBeans();

		for (GarbageCollectorMXBean garbageCollectorMXBean : list) {
			System.out.println("--MemoryManager: "
					+ garbageCollectorMXBean.getName());
			System.out.println("Collection count : "
					+ garbageCollectorMXBean.getCollectionCount());
			System.out.println("Collection time : "
					+ garbageCollectorMXBean.getCollectionTime());
			String arr[] = garbageCollectorMXBean.getMemoryPoolNames();
			for (String name : arr) {
				System.out.println("Memory Pool names: " + name);
			}
		}
	}

	private static void printMemoryPoolMXBean() {
		java.util.List<MemoryPoolMXBean> list = (List<MemoryPoolMXBean>) ManagementFactory
				.getMemoryPoolMXBeans();
		for (MemoryPoolMXBean temp : list) {
			MemoryPoolMXBean memoryPoolMXBean = temp;
			System.out.println("\n--MemoryManager: "
					+ memoryPoolMXBean.getName());
			System.out.println("Type = " + memoryPoolMXBean.getType());

			System.out.println("Usage = " + memoryPoolMXBean.getUsage());

			System.out
					.println("PeakUsage = " + memoryPoolMXBean.getPeakUsage());
			System.out.println("CollectionUsage = "
					+ memoryPoolMXBean.getCollectionUsage());
		}
	}

	/**
	 * Attention code uniquement compatible avec JDK 7 ne doit pas être appelé
	 * depuis autre JVM
	 * 
	 * @throws ClassNotFoundException
	 */
	private static void printBufferPoolMXBean() {
		try {
			MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
			Set<ObjectInstance> mbeans = mbs.queryMBeans(new ObjectName(
					"java.nio:type=BufferPool,*"), null);

			for (ObjectInstance bean : mbeans) {
				System.out.println("TypeBufferPool: "
						+ mbs.getAttribute(bean.getObjectName(), "Name"));
				System.out.println("Count:"
						+ mbs.getAttribute(bean.getObjectName(), "Count"));
				System.out.println("MemoryUsed:"
						+ mbs.getAttribute(bean.getObjectName(), "MemoryUsed"));
				System.out.println("Total capacity:"
						+ mbs.getAttribute(bean.getObjectName(),
								"TotalCapacity"));
			}
		} catch (SecurityException e) {
			System.out.println("Erreur de sécurité : " + e.getMessage());
		} catch (IllegalArgumentException e) {
			System.out.println("Erreur argument : " + e.getMessage());

		} catch (MalformedObjectNameException e) {
			System.out.println("Objet introuvable : " + e.getMessage());

		} catch (InstanceNotFoundException e) {
			System.out.println("Objet introuvable : " + e.getMessage());

		} catch (ReflectionException e) {
			System.out.println("Impossible d'appeler méthode BufferPool : "
					+ e.getMessage());

		} catch (MBeanException e) {
			System.out.println("Impossible d'appeler méthode BufferPool : "
					+ e.getMessage());

		} catch (AttributeNotFoundException e) {
			System.out.println("Impossible d'appeler méthode BufferPool : "
					+ e.getMessage());

		}
	}

	/**
	 * Bilan utilisation mémoire
	 */
	private static void printMemoryMXBean() {
		MemoryMXBean memoryMXBean = ManagementFactory.getMemoryMXBean();
		System.out.println("HeapMemoryUsage = "
				+ memoryMXBean.getHeapMemoryUsage());
		System.out.println("NonHeapMemoryUsage = "
				+ memoryMXBean.getNonHeapMemoryUsage());
		System.out.println("ObjectPendingFinalization = "
				+ memoryMXBean.getObjectPendingFinalizationCount());
	}

	public static void trace() {
		System.out.println("\n----------JVM Runtime Details---------");
		long maxMemory = Runtime.getRuntime().maxMemory();
		System.out.println("Maximum JVM Memory (-Xmx)   : "
				+ (maxMemory / (1024 * 1024)) + " MB");
		System.out.println("Total Used JVM Memory       : "
				+ (Runtime.getRuntime().totalMemory() / (1024 * 1024)) + " MB");

		System.out.println("\n----------GARBAGE COLLECTOR USAGES---------");
		printGarbageCollectorMXBean();

		System.out.println("\n----------MemoryPoolMXBean USAGES---------");
		printMemoryPoolMXBean();

		System.out.println("\n----------MemoryMXBean USAGES---------");
		printMemoryMXBean();

		String version = System.getProperty("java.specification.version");
		Float vers = Float.parseFloat(version);
		if (vers.compareTo(new Float(1.7)) <= 0) {
			String runtimeName = System.getProperty("java.runtime.name");
			if (runtimeName.startsWith("OpenJDK")) {
				System.out.println("\n----------Buffer Pool---------");
				printBufferPoolMXBean();
			}
		}
	}

}
