package fr.ippon.direct.cache;

import java.io.IOException;
import java.lang.reflect.Field;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import sun.misc.Unsafe;
import fr.ippon.direct.domain.Commune;

/**
 * @author IPPON_NANTES
 * 
 */
public class NativeMemoryCacheUnSafe {
	/**
	 * Création de la map pour stocker la clé ainsi que l'adresse de l'objet en
	 * mémoire native
	 */
	private Map<Long, Long> data = new HashMap<Long, Long>();

	/**
	 * Objet Unsafe pour accéder à la mémoire native
	 */
	private static Unsafe unsafe = null;

	/**
	 * Offset de stockage dans la mémoire native
	 */
	private static Long offset = new Long(0);

	static {
		try {
			unsafe = getUnsafeInstance();
		} catch (SecurityException e) {
			System.out.println("Exception de sécurité :" + e);
		} catch (NoSuchFieldException e) {
			System.out
					.println("Impossible d'accéder au champ de la classe Unsafe :"
							+ e);
		} catch (IllegalAccessException e) {
			System.out.println("Impossible d'accéder à la classe Unsafe :" + e);
		}
	}

	public void put(Long key, Commune com) throws IOException,
			SecurityException, NoSuchFieldException, IllegalArgumentException,
			IllegalAccessException {

		// on alloue 400 octets (longueur max en bytes)
		Long allocateMemory = unsafe.allocateMemory(400);
		
		unsafe.putObject(allocateMemory, offset, com);

		data.put(key, allocateMemory);

	}

	/**
	 * @param key
	 *            identifiant unique de l'objet
	 * @return objet à partir de la clef
	 * @throws ClassNotFoundException
	 * @throws IOException
	 * @throws SecurityException
	 * @throws NoSuchFieldException
	 * @throws IllegalArgumentException
	 * @throws IllegalAccessException
	 */
	public Commune get(Object key) throws ClassNotFoundException, IOException,
			SecurityException, NoSuchFieldException, IllegalArgumentException,
			IllegalAccessException {

		Long adresse = data.get(key);
		return (Commune) unsafe.getObject(adresse, offset);
	}

	/**
	 * Permet de lister le contenu de la Hashmap
	 * 
	 * @throws ClassNotFoundException
	 * @throws IOException
	 */
	public void print() throws ClassNotFoundException, IOException,
			SecurityException, IllegalArgumentException, NoSuchFieldException,
			IllegalAccessException {
		for (Iterator<Map.Entry<Long, Long>> it = data.entrySet().iterator(); it
				.hasNext();) {
			Map.Entry<Long, Long> e = (Entry<Long, Long>) it.next();
			System.out.println(get(e.getKey()));
		}
	}

	public void freeBuffer() throws ClassNotFoundException, IOException,
			SecurityException, IllegalArgumentException, NoSuchFieldException,
			IllegalAccessException {
		for (Iterator<Map.Entry<Long, Long>> it = data.entrySet().iterator(); it
				.hasNext();) {
			Map.Entry<Long, Long> e = (Entry<Long, Long>) it.next();
			Long adress = (Long) e.getValue();
			unsafe.freeMemory(adress);
			// adress = 0L;
		}
	}

	private static Unsafe getUnsafeInstance() throws SecurityException,
			NoSuchFieldException, IllegalAccessException {
		Field theUnsafeInstance = Unsafe.class.getDeclaredField("theUnsafe");
		theUnsafeInstance.setAccessible(true);
		return (Unsafe) theUnsafeInstance.get(Unsafe.class);
	}
}
